/*
 * 
 * * EMBL - European Bioinformatics institute
 * MSD Group
 * VAMSAS Project
 *
 * Created on Feb 22, 2006
 *
 * @author <a href="mailto:pierre@ebi.ac.uk">Pierre MARGUERITE</a>
 * @version  2  
 * @since 22 Feb. 2006
 */
package uk.ac.ebi.msd.vamsas.vamav.session;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.vamsas.client.simpleclient.SessionUrn;
import org.vamsas.client.simpleclient.VamsasSession;

import uk.ac.ebi.msd.vamsas.vamav.VAMAVClient;
import uk.ac.ebi.msd.vamsas.vamav.util.logger.VAMAVLogger;
import uk.ac.ebi.msd.vamsas.vamav.util.logger.VAMAVLoggerImpl;

/**
 * This class represents a VAMSAS session,
 * with a session ID, which can be opened and closed. 
 * @author <a href="mailto:pierre@ebi.ac.uk">Pierre MARGUERITE</a>
 * @since 22 Feb. 2006
 * @version 2 
 * 
 */
public class VAMAVSession extends VamsasSession {

    /**
     * ID of the current session
     */
	//private String sessionID = null;
		
		/**
		 * if the session is opened 
		 */
	private boolean open = false;
	
	/**
	 * if the current session has been initialised.
	 */
	private boolean init = false;
		
		/**
		 * log file containing session messages
		 * TODO merge with VAMSAS session log ?
		 */
	private File sessionLogFile = null;
	
	/**
	 * Path to the HTML page used to call the AstexViewer@MSD-
	 * EBI as applet
	 */
	private String appletFileName = null;
	
	/**
	 * Path to the Vamsas document file 
	 */
	private File vamsasDocumentFile = null;
		
		/**
		 * Logger to report messages
		 */
	private  VAMAVLogger logger = new VAMAVLoggerImpl(this.getClass());
	
	/**
	 * Structure grouping mode in use
	 */
	private int groupingMode = -1;
	
	/**
	 * List of attached VAMAV Client
	 * 
	 */
	private ArrayList<VAMAVClient> attachedClients = null;

	private SessionUrn sessionURN = null;

	/**
	 * Constructor specifying the session Directory
	 * 
	 * @param sessionDir session directory  containing all information relatives to the session
	 * @throws IOException if an error occurs during accessing to the session directory
	 */
	public VAMAVSession(File sessionDir) throws IOException
	 {
		super(sessionDir);
		sessionURN = new SessionUrn(this);
		this.logger.applidebug("setting logger seession");
					this.logger.setSession(this);
		}


		/**
		 * Retrieves the  sessionID attribut
		 *
		 * ID of the current session
		 *
		 * @return Returns the sessionID.
		 */
		public String getSessionID()
			{
				String id = null;
				if (this.sessionURN != null)
				{
					String urn = this.sessionURN.asFile().getAbsolutePath();
					//this.logger.applidebug("session urn "+urn);
					
					if ("\\".equals(File.separator))
						{
							//remove last separator at last position if found
							if (urn.charAt(urn.length() - 1) == '/')urn = urn.substring(0, urn.length() - 1);
							urn = urn.replaceAll("/", "\\\\");
							//this.logger.applidebug("session urn "+urn);
						}
					int index = urn.lastIndexOf(File.separator);
					
					if (index >-1)
					{//separator found, keep last part of the urn - filename
						id = urn.substring(index+1, urn.length());
					}
					else
					id = urn;
					//this.logger.applidebug("session ID "+urn);
				}
				return id;
			}


		 /**
		  * Sets sessionID attribut, ID of the current sesssion
		 *
		 * @param sessionID The sessionID to set.
		 */
	/*	public void setSessionID(String sessionID)
			{
				//this.sessionID = sessionID;
				this.logger.applidebug("setting logger seession");
				this.logger.setSession(this);
			}
*/

		/**
		 *specifies if the session 
		 *has been closed.
		 *
		 * @param sessionID The sessionID to set.
		 */
		public void setClosedSession( boolean closedSession)
			{
				//this.sessionID = sessionID;
				this.logger.applidebug("setting logger session");
				this.logger.setSession(this, closedSession);
			}


		/**
		 * Retrieves the  open attribut
		 *Indicates if the session has been opened.
		 *
		 * @return Returns the open.
		 */
		public boolean isOpen()
			{
				return this.open;
			}


		/**
		 *Sets open attribut
		 *
		 * @param open The open to set.
		 */
		public void sessionOpen()
			{
				this.open = true;
				this.init = true;
			}
		
		/**
		 *Sets open attribut
		 *
		 * @return true if the session has been opened then closed. false, otherwise.
		 */
		public boolean isClosed()
			{
				return !this.open;
			}
		
		/**
		 * finalize the object
		 */
		protected void finalize() throws Throwable 
			{
				try 
					{
						this.kill();
					} finally {
						super.finalize();
					}
			}
		
		/**
		 * kills the current session
		 * Stores required information
		 *
		 */
		private void kill()
			{
				if (this.attachedClients!= null)
					{
						for (int clientNumber = this.attachedClients.size() -1;clientNumber>=0; clientNumber-- )
							{
								VAMAVClient client = this.attachedClients.get(clientNumber) ;
								client = null;
							}
						this.attachedClients.clear();
						this.attachedClients = null;
					}
				/*if( this.sessionID != null)
					{
						this.sessionID = this.sessionID.replaceAll(this.sessionID, "");
						this.sessionID = null;
					}*/
			}
		
		/**
		 * Retrieves the  attachedClients attribut, list of client
		 * associated to the current session
		 *
		 * @return Returns the attachedClients.
		 */
		private ArrayList<VAMAVClient> getAttachedClients()
			{
				return this.attachedClients;
			}
		
		/**
		 * Retrieves an  attachedClient at a given index
		 *
		 *
		 * @param index index of the client to retrieved
		 * @return the attached client at the given index, if found. null otherwise.
		 */
		public VAMAVClient getAttachedClient(int index )
			{
				if (this.getAttachedClients() == null || index<0 || index>this.getAttachedClients().size())
					return null;
				return this.getAttachedClients().get(index);
			}
		
		/**
		 * Attached the client to a VAMAV Session
		 * 
		 * @param client client to add the current session.
		 */
		public void addAttachedClient(VAMAVClient client)
			{
				if( this.getAttachedClients() == null)
					{
						this.attachedClients = new ArrayList<VAMAVClient>();
					}
				this.getAttachedClients().add(client);
				client.setVAMAVSession(this);
			}

		/**
		 * removes a client from the current session
		 * 
		 * @param client client to  remove
		 */
		public void removeClient(VAMAVClient client)
			{
				if( this.getAttachedClients() == null )
					{
						logger.appliError("Unable to remove client. There is no client attached.");
					}
				this.getAttachedClients().remove(client);
				if ( this.getAttachedClients().size()<1)
					{
						this.close();
					}
			}

		/**
		 * Retrieves the  sessionDirectory attribut
		 * 
		 * retrieves the session directory containing files relatives to the session
		 *
		 * @return Returns the sessionDirectory.
		 */
		public File getSessionDirectory()
			{
				return this.sessionDir;
			}


		/**
		 *Sets sessionDirectory attribut
		 *
		 *Sets the path to the session directory
		 *
		 * @param sessionDirectory The sessionDirectory to set.
		 */
		public void setSessionDirectory(File sessionDirectory)
			{
				if( sessionDirectory!= null)
					{
						if(!sessionDirectory.exists()) 
							if(  !sessionDirectory.mkdirs())
								{
									logger.sessionError("Unable to create session directory.");
								}
						this.sessionDir = sessionDirectory;
						if (!this.logger.isProduction()) this.logger.applidebug("session directory "+sessionDirectory.getAbsolutePath());
						
					}
			}

		/**
		 * Retrieves the  sessionLogFile attribut
		 *
		 *Path the session log file containing reported messages relatives
		 *to the session
		 *
		 * @return Returns the sessionLogFile.
		 */
		public File getSessionLogFile()
			{
				return this.sessionLogFile;
			}


		/**
		 *Sets sessionLogFile attribut
		 *
		 *Sets the file containing all reported session messages
		 *
		 * @param sessionLogFile The sessionLogFile to set.
		 */
		public void setSessionLogFile(File sessionLogFile)
			{
				this.sessionLogFile = sessionLogFile;
				if (sessionLogFile != null && !this.logger.isProduction())
					{
						this.logger.applidebug(sessionLogFile.getAbsolutePath());
					}
				
			}
		
		/**
		 * Closes the current session
		 *Clears client list //TODO close attached clients
		 *
		 */
		public void close ()
			{
				logger.sessionInfo("Closing session "+this.getSessionID()                                                                                            );
				this.open = false;
				if (this.getAttachedClients() != null)
					{
						this.getAttachedClients().clear();
						this.attachedClients = null;
					}
			}


		/**
		 * Retrieves the  appletFileName attribut
		 *name of the file containing html page to call the AstexViewer@MSD-EBI
		 *
		 * @return Returns the appletFileName. 
		 */
		public String getAppletFileName()
			{
				return this.appletFileName;
			}
		
		/**
		 * Retrieves the  appletFile object
		 *
		 * @return Returns the appletFile object
		 */
		public File getAppletFile()
			{
				return new File (this.getSessionDirectory().getAbsolutePath()+File.separator+this.appletFileName);
			}


		/**
		 *Sets appletFileName attribut
		 *
		 * @param appletFileName The appletFileName to set.
		 */
		public void setAppletFileName(String appletFileName)
			{
				this.appletFileName = appletFileName;
			}


		/**
		 * Retrieves the  vamsasDocument File 
		 *
		 * @return Returns the vamsasDocumentFile.
		 */
		public File getVamsasDocumentFile()
			{
				return this.vamsasDocumentFile;
			}


		/**
		 *Sets vamsasDocumentFile associated to /treated in the session
		 *
		 * @param vamsasDocumentFile The vamsasDocumentFile to set.
		 */
		public void setVamsasDocumentFile(File vamsasDocumentFile)
			{
				this.vamsasDocumentFile = vamsasDocumentFile;
			}


		/**
		 * Retrieves the  groupingMode currently used in the 
		 * session for process data
		 *
		 * @return Returns the groupingMode.
		 */
		public int getGroupingMode()
			{
				return this.groupingMode;
			}


		/**
		 *Sets groupingMode attribut
		 *
		 * @param groupingMode The groupingMode to set.
		 */
		public void setGroupingMode(int groupingMode)
			{
				this.groupingMode = groupingMode;
			}


		/**
		 * Retrieves the  init attribut
		 * Indicates if the current session has been initialised
		 * @return Returns the init.
		 */
		public boolean isInit()
			{
				return this.init;
			}
}
